package com.ruoyi.system.service.impl;

import java.util.List;

import cn.hutool.core.util.ObjectUtil;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.domain.entity.AsUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.uuid.IdUtils;
import com.ruoyi.system.domain.*;
import com.ruoyi.system.mapper.AsUserMapper;
import com.ruoyi.system.mapper.EtCouponClaimLogMapper;
import com.ruoyi.system.mapper.EtOrderMapper;
import com.ruoyi.system.service.CallbackService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.system.mapper.EtCouponMapper;
import com.ruoyi.system.service.IEtCouponService;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;

/**
 * 优惠券Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-08-05
 */
@Service
public class EtCouponServiceImpl implements IEtCouponService
{
    @Resource
    private EtCouponMapper etCouponMapper;

    @Resource
    private EtCouponClaimLogMapper etCouponClaimLogMapper;

    @Resource
    private AsUserMapper userMapper;

    @Resource
    private EtOrderMapper etOrderMapper;

    @Autowired
    private CallbackService callbackService;

    /**
     * 查询优惠券
     *
     * @param couponId 优惠券主键
     * @return 优惠券
     */
    @Override
    public EtCoupon selectEtCouponByCouponId(Long couponId)
    {
        return etCouponMapper.selectEtCouponByCouponId(couponId);
    }

    /**
     * 查询优惠券列表
     *
     * @param etCoupon 优惠券
     * @return 优惠券
     */
    @Override
    public List<EtCoupon> selectEtCouponList(EtCoupon etCoupon)
    {
        return etCouponMapper.selectEtCouponList(etCoupon);
    }

    /**
     * 新增优惠券
     *
     * @param etCoupon 优惠券
     * @return 结果
     */
    @Override
    public int insertEtCoupon(EtCoupon etCoupon)
    {
        etCoupon.setCreateTime(DateUtils.getNowDate());
        return etCouponMapper.insertEtCoupon(etCoupon);
    }

    /**
     * 修改优惠券
     *
     * @param etCoupon 优惠券
     * @return 结果
     */
    @Override
    public int updateEtCoupon(EtCoupon etCoupon)
    {
        return etCouponMapper.updateEtCoupon(etCoupon);
    }

    /**
     * 批量删除优惠券
     *
     * @param couponIds 需要删除的优惠券主键
     * @return 结果
     */
    @Override
    public int deleteEtCouponByCouponIds(Long[] couponIds)
    {
        return etCouponMapper.deleteEtCouponByCouponIds(couponIds);
    }

    /**
     * 删除优惠券信息
     *
     * @param couponId 优惠券主键
     * @return 结果
     */
    @Override
    public int deleteEtCouponByCouponId(Long couponId)
    {
        return etCouponMapper.deleteEtCouponByCouponId(couponId);
    }

    /**
     * 优惠券购买
     * @param userId
     * @param couponId
     * @return
     */
    @Override
    @Transactional
    public Boolean couponBuy(Long userId, Long couponId) {
        /**
         *   购买分为抵用券订单，会员卡订单、折扣卡订单
         *   1. 插入一条记录，et_coupon_user_log  获取方式是购买
         *   2. 增加订单记录，类型是优惠券订单
         *   3. 增加账变, 所属人为运营商
         * */
        // 1. 插入一条记录，et_coupon_user_log  获取方式是购买
        AsUser asUser = userMapper.selectUserById(userId);
        EtCoupon etCoupon = etCouponMapper.selectEtCouponByCouponId(couponId);
        EtCouponClaimLog etCouponClaimLog = new EtCouponClaimLog();
        etCouponClaimLog.setCouponId(couponId);
        etCouponClaimLog.setUserId(userId);
        etCouponClaimLog.setUserName(asUser.getUserName());
        etCouponClaimLog.setCreateTime(DateUtils.getNowDate());
        etCouponClaimLog.setGainMethod(ServiceConstants.COUPON_GAIN_METHOD_BUY);
        etCouponClaimLog.setAreaId(etCoupon.getAreaId());
        etCouponClaimLog.setAreaName(etCoupon.getAreaName());
        int i1 = etCouponClaimLogMapper.insertEtCouponClaimLog(etCouponClaimLog);
        if(i1 == 0){
            throw new ServiceException("保存优惠券领取记录失败");
        }

        // 2. 增加订单记录，类型是优惠券订单
        EtOrder order = createOrder(etCoupon, asUser);
        if(ObjectUtil.isNotNull(order)){
            int i = etOrderMapper.insertEtOrder(order);
            if(i > 0){
                // 账变是针对账户而言的，优惠券充值后的入账是进入到那个账户？  有效期    季度  quarter
                callbackService.capitalFlowRecords(order,ServiceConstants.FLOW_TYPE_INCOME,
                        ServiceConstants.ORDER_TYPE_COUPON_INCOME,ServiceConstants.OWNER_TYPE_OPERATOR,null,ServiceConstants.PAY_TYPE_WX);
                return true;
            }
        }
        return false;
    }


    private EtOrder createOrder(EtCoupon etCoupon,AsUser user) {
        EtOrder etOrder = new EtOrder();
        etOrder.setOrderNo(IdUtils.getOrderNo("wx"));
        etOrder.setUserId(user.getUserId());
        etOrder.setUserName(user.getUserName());
        etOrder.setMark("优惠券购买");
        etOrder.setCreateTime(DateUtils.getNowDate());
        etOrder.setAreaId(etCoupon.getAreaId());
        /** 订单类型 1骑行 2预约 3套餐 4押金 根据类型不同下单的参数也不同，不同的参数有： 金额、类型、rule_id（套餐id）*/
        etOrder.setArea(etCoupon.getAreaName());
        etOrder.setType(ServiceConstants.ORDER_TYPE_COUPON);
        etOrder.setTotalFee(etCoupon.getSnackPrice());
        etOrder.setPayFee(etCoupon.getSnackPrice());
        return etOrder;
    }
}
